@@ -1,6 +1,6 @@
 // Filename: file.rs
-// Version:	 0.2
-// Date:	 24-09-2021 (DD-MM-YYYY)
+// Version:	 0.3
+// Date:	 01-01-2022 (DD-MM-YYYY)
 // Library:  gpcas_base
 //
 // Copyright (c) 2021 Kai Rese
@@ -81,20 +81,24 @@ pub trait GpcasFileStruct: 'static + DeserializeOwned + Serialize {
 
 /// A file type which has support for serialization and deserialization.
 pub enum FileEncodingType {
-    /// Uses the `bincode` crate for serialization and deserialization.
+    /// The file is serialized to binary by the `bincode` crate.
     Binary,
     /// The file has a JSON format.
     Json,
+    /// The file has a TOML format.
+    Toml,
 }
 
 /// Data of a GPCAS file struct.
 ///
 /// This is used so already parsed data, such as JSON objects, can be used for deserialization.
 pub enum EncodedStructData<'a> {
-    /// The data is encoded in binary.
+    /// The data is encoded in binary compatible to the `bincode` crate.
     Binary(&'a [u8]),
     /// The data is encoded as a parsed JSON object.
     Json(serde_json::Value),
+    /// The data is encoded as a parsed TOML value.
+    Toml(toml::Value),
 }
 
 /// The representation and structure of a gpcas file. Wraps the content behind a header.
@@ -198,6 +202,48 @@ pub fn deserialize<T: GpcasFileStruct>(
                 Err(DeserializationError::NoGpcasFile)
             }
         }
+        FileEncodingType::Toml => {
+            if let Ok(wrapper) = toml::from_slice::<toml::Value>(raw) {
+                if wrapper.is_table()
+                    && wrapper.as_table().unwrap().len() == 3
+                    && wrapper.as_table().unwrap().contains_key("identifier")
+                    && wrapper.as_table().unwrap().contains_key("version")
+                    && wrapper.as_table().unwrap().contains_key("data")
+                    && wrapper.as_table().unwrap()["identifier"].is_str()
+                    && wrapper.as_table().unwrap()["version"].is_integer()
+                    && wrapper.as_table().unwrap()["data"].is_table()
+                {
+                    let wrapper = wrapper.as_table().unwrap();
+                    if wrapper["identifier"].as_str().unwrap() != T::FILE_IDENTIFIER {
+                        return Err(DeserializationError::WrongFileType(
+                            T::FILE_IDENTIFIER,
+                            wrapper["identifier"].as_str().unwrap().to_string(),
+                        ));
+                    }
+
+                    let version = wrapper["version"].as_integer().unwrap() as usize;
+                    if version == T::CURRENT_FILE_VERSION {
+                        wrapper["data"].clone().try_into()
+                            .map_err(|e| DeserializationError::CorruptFileData(Box::new(e)))
+                    } else if let Ok(index) =
+                    T::COMPATIBLE_VERSIONS.binary_search_by_key(&version, |(i, _)| *i)
+                    {
+                        T::COMPATIBLE_VERSIONS[index].1(EncodedStructData::Toml(
+                            wrapper["data"].clone(),
+                        ))
+                            .map_err(|e| DeserializationError::CorruptFileData(e))
+                    } else {
+                        Err(DeserializationError::IncompatibleFileVersion(
+                            wrapper["version"].as_integer().unwrap() as usize,
+                        ))
+                    }
+                } else {
+                    Err(DeserializationError::NoGpcasFile)
+                }
+            } else {
+                Err(DeserializationError::NoGpcasFile)
+            }
+        }
     }
 }
 
@@ -219,6 +265,9 @@ where
         EncodedStructData::Json(value) => Ok(serde_json::from_value::<Old>(value)
             .map_err(|e| Box::new(e) as Box<dyn std::error::Error>)?
             .into()),
+        EncodedStructData::Toml(value) => Ok(value.try_into::<Old>()
+            .map_err(|e| Box::new(e) as Box<dyn std::error::Error>)?
+            .into()),
     }
 }
 
@@ -247,6 +296,22 @@ pub fn serialise<T: GpcasFileStruct>(from: &T, encoding_type: FileEncodingType)
             );
             serde_json::to_vec_pretty(&serde_json::Value::Object(map)).unwrap()
         }
+        FileEncodingType::Toml => {
+            let mut map = toml::value::Table::new();
+            map.insert(
+                "identifier".to_string(),
+                toml::Value::String(T::FILE_IDENTIFIER.to_string()),
+            );
+            map.insert(
+                "version".to_string(),
+                toml::Value::Integer(T::CURRENT_FILE_VERSION as i64),
+            );
+            map.insert(
+                "data".to_string(),
+                toml::Value::try_from(from).unwrap()
+            );
+            toml::to_string_pretty(&toml::Value::Table(map)).unwrap().into_bytes()
+        }
     }
 }
 
